@@ -114,7 +114,7 @@
 ;;; uninterned.
 ;;; Additionally, you can specify an arbitrary way to destroy
 ;;; random bootstrap stuff on per-package basis.
-(defun !unintern-init-only-stuff (&aux result)
+(defun !unintern-init-only-stuff ()
   (dolist (package (list-all-packages))
     (sb-int:awhen (find-symbol "!REMOVE-BOOTSTRAP-SYMBOLS" package)
       (funcall sb-int:it)))
@@ -142,16 +142,13 @@
     ;; So loop over all structure classoids and clobber any
     ;; symbol that should be uninternable.
     (maphash (lambda (classoid layout)
-               (when (structure-classoid-p classoid)
-                 (let ((dd (layout-%info layout)))
-                   (setf (dd-constructors dd)
-                         (delete-if (lambda (x)
-                                      (and (consp x) (uninternable-p (car x))))
-                                    (dd-constructors dd))))))
+               (declare (ignore classoid))
+               (binding* ((dd (layout-info layout) :exit-if-null))
+                 (setf (dd-constructors dd)
+                       (delete-if (lambda (x) (and (consp x) (uninternable-p (car x))))
+                                  (dd-constructors dd)))))
              (classoid-subclasses (find-classoid t)))
-    ;; PATHNAME is not a structure-classoid
-    (setf (sb-kernel:dd-constructors (sb-kernel:find-defstruct-description 'pathname))
-          nil)
+
     ;; Todo: perform one pass, then a full GC, then a final pass to confirm
     ;; it worked. It should be an error if any uninternable symbols remain,
     ;; but at present there are about 7 symbols with referrers.
@@ -186,7 +183,7 @@
     (setf (sb-c::vop-parse-body v) nil))
   ;; Used for inheriting from other VOPs, not needed in the target.
   (setf sb-c::*backend-parsed-vops* (make-hash-table))
-  result)
+  nil)
 
 
 ;;; Check for potentially bad format-control strings
@@ -333,7 +330,6 @@ Please check that all strings which were not recognizable to the compiler
   ;; SAVE-LISP-AND-DIE.
   #-sb-devel (!unintern-init-only-stuff)
 
-
   (do-all-symbols (symbol)
     ;; Don't futz with the header of static symbols.
     ;; Technically LOGIOR-HEADER-BITS can only be used on an OTHER-POINTER-LOWTAG
@@ -402,29 +398,33 @@ Please check that all strings which were not recognizable to the compiler
         (let ((name (string symbol)))
           (and (> (length name) 2)
                (string= name "M:" :end1 2)))))
+#-sb-devel
 (let ((counts
        (mapcar (lambda (x)
                  (list x
                        (sb-impl::package-external-symbol-count x)
                        (sb-impl::package-internal-symbol-count x)))
-               (sort (list-all-packages) #'string< :key 'package-name))))
-  #-sb-devel
+               (sort (list-all-packages) #'string< :key 'package-name)))
+      (precious-internals
+       ((lambda (list &aux (ht (make-hash-table)))
+          (dolist (sym list ht) (setf (gethash sym ht) t)))
+        '(sb-alien::alien-callback-p sb-alien::alien-lambda ; the API uses internals, really?
+          sb-c::tab sb-c::scramble ; for perfecthash
+          sb-c::make-transform ; cl-protobufs uses this
+          sb-impl::%default-comma-constructor
+          sb-kernel::%%make-random-state
+          sb-lockless::+hash-nbits+ sb-lockless::%make-so-set-node ; for tests
+          sb-loop::*loop-epilogue* sb-loop::add-loop-path ; internals to keep CLSQL working
+          sb-profile::make-counter)))) ; for a test
   ;; Remove inline expansions
   (do-symbols (symbol #.(find-package "SB-C"))
     (when (equal (symbol-package symbol) #.(find-package "SB-C"))
       (sb-int:clear-info :function :inlining-data symbol)
       (sb-int:clear-info :function :inlinep symbol)))
   (sb-impl::shake-packages
-   ;; Development mode: retain all symbols with any system-related properties
-   #+sb-devel
-   (lambda (symbol accessibility)
-     (declare (ignore accessibility))
-     (or (sb-kernel:symbol-%info symbol)
-         (sb-kernel:%symbol-function symbol)
-         (and (boundp symbol) (not (keywordp symbol)))))
-   ;; Release mode: retain all symbols satisfying this intricate test
-   #-sb-devel
    (lambda (symbol accessibility)
+    (or
+     (gethash symbol precious-internals) ; prevails over any condition below
      (case (symbol-package symbol)
       (#.(find-package "SB-VM")
        (or (eq accessibility :external)
@@ -436,33 +436,22 @@ Please check that all strings which were not recognizable to the compiler
                             ;; need this for defining a vop which
                             ;; tests the x86-64 allocation profiler
                             sb-vm::pseudo-atomic
-                            ,@(or #+(or x86 x86-64) '(sb-vm::%vector-cas-pair
-                                                      sb-vm::%instance-cas-pair
-                                                      sb-vm::%cons-cas-pair))
+                            ,@(or #+(or arm64 x86 x86-64)
+                                  '(sb-vm::%vector-cas-pair
+                                    sb-vm::%instance-cas-pair
+                                    sb-vm::%cons-cas-pair))
                             ;; Naughty outside-world code uses these.
                             #+x86-64 sb-vm::reg-in-size))
            (let ((s (string symbol))) (and (search "THREAD-" s) (search "-SLOT" s)))
            (search "-OFFSET" (string symbol))
            (search "-TN" (string symbol))))
-      (#.(find-package "SB-ALIEN")
-       (or (eq accessibility :external) (eq symbol 'sb-alien::alien-callback-p)))
       (#.(mapcar 'find-package
-                 '("SB-ASSEM" "SB-BROTHERTREE" "SB-DISASSEM" "SB-FORMAT"
-                   "SB-IMPL" "SB-KERNEL" "SB-MOP" "SB-PCL" "SB-PRETTY" "SB-PROFILE"
+                 '("SB-ALIEN" "SB-ASSEM" "SB-BROTHERTREE" "SB-C" "SB-DISASSEM" "SB-FORMAT"
+                   "SB-IMPL" "SB-KERNEL" "SB-LOCKLESS" "SB-LOOP" "SB-MOP" "SB-PCL"
+                   "SB-PRETTY" "SB-PROFILE"
                    "SB-REGALLOC" "SB-SYS" "SB-UNICODE" "SB-UNIX" "SB-WALKER"))
        ;; Assume all and only external symbols must be retained
          (eq accessibility :external))
-      (#.(find-package "SB-C")
-       (or (eq accessibility :external)
-           (member symbol '(sb-c::tab sb-c::scramble))))
-      (#.(find-package "SB-LOOP")
-       (or (eq accessibility :external)
-           ;; Retain some internals to keep CLSQL working.
-           (member symbol '(sb-loop::*loop-epilogue*
-                            sb-loop::add-loop-path))))
-      (#.(find-package "SB-LOCKLESS")
-       (or (eq accessibility :external)
-           (member symbol '(sb-lockless::+hash-nbits+)))) ; for a test
       (#.(find-package "SB-THREAD")
        (or (eq accessibility :external)
            ;; for some reason a recent change caused the tree-shaker to drop MAKE-SPINLOCK
@@ -490,7 +479,7 @@ Please check that all strings which were not recognizable to the compiler
            ;; By default, retain any symbol with any attachments
            (or (sb-kernel:symbol-%info symbol)
                (sb-kernel:%symbol-function symbol)
-               (and (boundp symbol) (not (keywordp symbol))))))))
+               (and (boundp symbol) (not (keywordp symbol)))))))))
    :verbose nil :print nil)
   (unintern 'sb-impl::shake-packages 'sb-impl)
   (let ((sum-delta-ext 0)
@@ -548,7 +537,7 @@ Please check that all strings which were not recognizable to the compiler
 
 (setq sb-c:*compile-to-memory-space* :auto)
 (when (find-package "SB-INTERPRETER") (setq sb-ext:*evaluator-mode* :interpret))
-#+x86-64 (sb-ext:fold-identical-code :aggressive t :preserve-docstrings t)
+#+(and x86-64 (not sb-devel)) (sb-ext:fold-identical-code :aggressive t :preserve-docstrings t)
 
 ;; See comments in 'readtable.lisp'
 (setf (readtable-base-char-preference *readtable*) :symbols)
