@@ -945,7 +945,7 @@ NOTE: This interface is experimental and subject to change."
 (declaim (inline swapped-args-fun))
 (defun swapped-args-fun (fun)
   (declare (type function fun))
-  (lambda (x y)
+  (named-lambda "FUN" (x y) ; prevent the lambda from retaining #:SWAPPED-ARGS-FUN
     (funcall fun y x)))
 
 ;;; Return the numeric value of a type bound, i.e. an interval bound
@@ -1457,6 +1457,7 @@ NOTE: This interface is experimental and subject to change."
 
 (in-package "SB-KERNEL")
 
+(declaim (inline fp-zero-p))
 (defun fp-zero-p (x)
   (typecase x
     (single-float (zerop x))
@@ -1509,19 +1510,16 @@ NOTE: This interface is experimental and subject to change."
     (cons nil)
     (t t)))
 
-(declaim (inline first-bit-set))
-(defun first-bit-set (x)
-  #+(and x86-64 (not sb-xc-host))
-  (truly-the (values (mod #.sb-vm:n-word-bits) &optional)
-             (%primitive sb-vm::unsigned-word-find-first-bit (the word x)))
-  #-(and x86-64 (not sb-xc-host))
-  (1- (integer-length (logand x (- x)))))
+(defun count-trailing-zeros (integer)
+  (let ((integer (ldb (byte sb-vm:n-word-bits 0) integer)))
+    (integer-length (ldb (byte sb-vm:n-word-bits 0) (1- (logand integer (- integer)))))))
 
 (defun integer-float-p (float)
   (and (floatp float)
        (multiple-value-bind (significand exponent) (integer-decode-float float)
          (or (plusp exponent)
-             (<= (- exponent) (first-bit-set significand))))))
-
+             (<= (- exponent)
+                 ;; count-trailing-zeros, but wider for 32-bit platforms
+                 (integer-length (ldb (byte 64 0) (lognor significand (- significand)))))))))
 
 (defvar *top-level-form-p* nil)
