@@ -48,7 +48,7 @@
 ;;;    (foo))
 ;;; provided that the first branch is true "often enough".
 
-;;; These 4 symbols are initialized by create_thread_struct()
+;;; These 4 symbols are initialized by alloc_thread_struct()
 (defvar *interrupts-enabled*)
 (defvar *interrupt-pending*)
 #+sb-safepoint (defvar *thruption-pending*)
@@ -65,7 +65,7 @@
 ;;; would not cut it, as upon leaving WITHOUT-INTERRUPTS the pending
 ;;; handlers is run with stuff from the function in which this is
 ;;; still on the stack.
-(defparameter *unblock-deferrables-on-enabling-interrupts-p* nil)
+(defvar *unblock-deferrables-on-enabling-interrupts-p* nil)
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
   (dolist (symbol '(*unblock-deferrables-on-enabling-interrupts-p*
