@@ -556,8 +556,7 @@
                       (:constructor make-component
                        (head tail &aux (last-block tail)
                                        (outer-loop
-                                        (make-loop :outer head (list tail)))))
-                      #-sb-xc-host :no-constructor-defun)
+                                        (make-loop :outer head (list tail))))))
   ;; space where this component will be allocated in
   ;; :auto won't make any codegen optimizations pertinent to immobile space,
   ;; but will place the code there given sufficient available space.
@@ -695,7 +694,8 @@
   (kind (missing-arg)
         :type (member :special-bind :catch :unwind-protect
                       :block :tagbody :dynamic-extent
-                      #-c-stack-is-control-stack :restore-nsp))
+                      #-c-stack-is-control-stack :restore-nsp)
+        :read-only t)
   ;; the node that messes things up. This is the last node in the
   ;; non-messed-up environment. Null only temporarily. This could be
   ;; deleted due to unreachability.
@@ -713,8 +713,7 @@
 
 ;;; The ENVIRONMENT structure represents the result of environment analysis.
 (defstruct (environment (:copier nil)
-                        (:constructor make-environment (lambda))
-                        #-sb-xc-host :no-constructor-defun)
+                        (:constructor make-environment (lambda)))
   ;; the function that allocates this environment
   (lambda (missing-arg) :type clambda :read-only t)
   ;; a list of all the LAMBDA-VARs and NLX-INFOs needed from enclosing
@@ -764,8 +763,7 @@
 ;;; ENVIRONMENT-NLX-INFO.
 (defstruct (nlx-info
             (:copier nil)
-            (:constructor make-nlx-info (cleanup block))
-            #-sb-xc-host :no-constructor-defun)
+            (:constructor make-nlx-info (cleanup block)))
   ;; the cleanup associated with this exit. In a catch or
   ;; unwind-protect, this is the :CATCH or :UNWIND-PROTECT cleanup,
   ;; and not the cleanup for the escape block. The CLEANUP-KIND of
@@ -1331,8 +1329,7 @@
 (defstruct (optional-dispatch
             (:include functional) (:copier nil)
             (:constructor make-optional-dispatch
-                (arglist allowp keyp %source-name %debug-name source-path))
-            #-sb-xc-host :no-constructor-defun)
+                (arglist allowp keyp %source-name %debug-name source-path)))
   ;; the original parsed argument list, for anyone who cares
   (arglist nil :type list)
   ;; true if &ALLOW-OTHER-KEYS was supplied
@@ -1534,7 +1531,10 @@
   (same-refs nil :type (or null cons)))
 (defprinter (ref :identity t)
   (%source-name :test (neq %source-name '.anonymous.))
-  leaf)
+  (leaf :prin1 (if (and (constant-p leaf)
+                        (not (symbolp (constant-value leaf))))
+                   (constant-value leaf)
+                   leaf)))
 
 (defstruct (multiple-successors-node
             (:constructor nil)
@@ -1563,8 +1563,7 @@
 
 (defstruct (jump-table (:include multiple-successors-node)
                        (:constructor make-jump-table (index))
-                       (:copier nil)
-                       #-sb-xc-host :no-constructor-defun)
+                       (:copier nil))
   (index (missing-arg) :type lvar)
   (targets nil :type list))
 
@@ -1577,8 +1576,7 @@
                  (:conc-name set-)
                  (:predicate set-p)
                  (:constructor make-set (var value))
-                 (:copier nil)
-                 #-sb-xc-host :no-constructor-defun)
+                 (:copier nil))
   ;; descriptor for the variable set
   (var (missing-arg) :type basic-var)
   ;; LVAR for the value form
@@ -1643,7 +1641,10 @@
   (pass-nargs t :type boolean)
   (or-chain-computed nil :type boolean))
 (defprinter (combination :identity t)
-  (fun :prin1 (lvar-uses fun))
+  (fun :prin1 (let ((uses (lvar-uses fun)))
+                (or (and (ref-p uses)
+                         (ref-fun-name uses t))
+                    uses)))
   (args :prin1 (mapcar (lambda (x)
                          (if x
                              (lvar-uses x)
@@ -1655,8 +1656,7 @@
 ;;; receiving forms.
 (defstruct (mv-combination (:include basic-combination)
                            (:constructor make-mv-combination (fun))
-                           (:copier nil)
-                           #-sb-xc-host :no-constructor-defun))
+                           (:copier nil)))
 (defprinter (mv-combination)
   (fun :prin1 (lvar-uses fun))
   (args :prin1 (mapcar #'lvar-uses args)))
@@ -1680,8 +1680,7 @@
                     (:conc-name return-)
                     (:predicate return-p)
                     (:constructor make-return (result lambda))
-                    (:copier nil)
-                    #-sb-xc-host :no-constructor-defun)
+                    (:copier nil))
   ;; the lambda we are returning from. Null temporarily during
   ;; ir1tran.
   (lambda nil :type (or clambda null))
@@ -1702,8 +1701,7 @@
 (defstruct (cast (:include valued-node)
                  (:copier nil)
                  (:constructor %make-cast
-                     (asserted-type type-to-check value derived-type context))
-                 #-sb-xc-host :no-constructor-defun)
+                     (asserted-type type-to-check value derived-type context)))
   (asserted-type (missing-arg) :type ctype)
   (type-to-check (missing-arg) :type ctype)
   ;; an indication of what we have proven about how this type
