@@ -17,8 +17,6 @@
 ;;; attempt
 (defvar *pack-assign-costs* t)
 (defvar *pack-optimize-saves* t)
-;;; FIXME: Perhaps SB-FLUID should be renamed to SB-TWEAK and these
-;;; should be made conditional on SB-TWEAK.
 
 (declaim (ftype (function (component) index) ir2-block-count))
 
@@ -351,6 +349,59 @@
      (t
       `("~2D: not referenced?" ,loc)))))
 
+(eval-when (:compile-toplevel)
+  (assert (zerop (rem sb-vm:finite-sc-offset-limit 8)))) ; multiple of 8
+
+(defmacro do-sc-locations ((location locations &optional result (increment 1))
+                           &body body)
+  (let ((inc '#:increment)
+        (bitmap '#:bits)
+        (bias '#:bias))
+    (declare (ignorable inc))
+    `(let ((,bitmap ,locations)
+           (,inc (truly-the (member 1 2 4) ,increment))
+           (,bias 0))
+       ;; This bitmap won't be a machine word if #+sparc but who cares?
+       (declare (type sb-vm:finite-sc-offset-map ,bitmap))
+       ;; BIAS can take on the limiting value at its final iteration
+       (declare (type (integer 0 ,sb-vm:finite-sc-offset-limit) ,bias))
+       (block nil
+         #|
+         ;; This should be faster since there is no loop nesting
+         ;; and only the 1 bits are visited. But it didn't benchmark better.
+         ;; Must dig deeper.
+         #+(and (or x86 x86-64) (not sb-xc-host))
+         (unless (zerop ,bitmap)
+           (loop named #:noname
+                 do
+             (let ((.index. (count-trailing-zeros ,bitmap)))
+               ;; BIAS is the bit index in the original LOCATIONS map
+               ;; which is at index 0 in BITMAP
+               (let ((,location (truly-the sb-vm:finite-sc-offset (+ .index. ,bias))))
+                 ,@body
+                 ;; Shift using 2 steps due to the type constraint on digit-logical-shift-right
+                 ;; and/or CPU behavior of oversized shift.
+                 ;; Consider: 32 bit word and bit 31 (and only that) is on. After processing it,
+                 ;; we need to shift out 32 bits to get to the "next" bit. There is no next bit,
+                 ;; so the word should become 0. But shifting by 32 = shifting by 0 which won't do
+                 ;; the right thing.
+                 (setq ,bitmap (sb-bignum:%digit-logical-shift-right
+                                 (sb-bignum:%digit-logical-shift-right ,bitmap .index.)
+                                 ,inc))
+                 (when (zerop ,bitmap) (loop-finish))
+                 (incf ,bias (truly-the (integer 0 ,sb-vm:n-word-bits)
+                                        (+ .index. ,inc)))))))
+         #-(and (or x86 x86-64) (not sb-xc-host))
+         |#
+         (loop named #:outer repeat (/ sb-vm:finite-sc-offset-limit 8)
+               do (when (ldb-test (byte 8 ,bias) ,bitmap)
+                    ;; scan 8 bits starting at BIAS
+                    (loop named #:inner
+                          for ,location from ,bias to (+ ,bias 7) by ,inc
+                          when (logbitp ,location ,bitmap) do (progn ,@body)))
+                  (incf ,bias 8))
+         ,result))))
+
 ;;; If load TN packing fails, try to give a helpful error message. We
 ;;; find a TN in each location that conflicts, and print it.
 (defun failed-to-pack-load-tn-error (scs op)
@@ -418,7 +469,6 @@
 
 ;;;; register saving
 
-#-sb-devel
 (declaim (start-block optimized-emit-saves emit-saves assign-tn-costs
                       pack-save-tn))
 
@@ -488,11 +538,14 @@
     (when (eq (tn-kind save) :specified-save)
       (setf (tn-kind save) :save))
     (aver (eq (tn-kind save) :save))
-    (emit-operand-load node block tn save
-                       (if (eq (vop-info-move-args (vop-info vop))
-                               :local-call)
-                           (reverse-find-vop 'allocate-frame vop)
-                           vop))
+    (multiple-value-bind (before block)
+        (if (eq (vop-info-move-args (vop-info vop)) :local-call)
+            (let ((before (reverse-find-vop 'allocate-frame vop)))
+              ;; Because of SPLIT-IR2-BLOCKS the ALLOCATE-FRAME VOP
+              ;; may be in a different block.
+              (values before (vop-block before)))
+            (values vop block))
+      (emit-operand-load node block tn save before))
     (emit-operand-load node block save tn next)))
 
 ;;; Return a VOP after which is an OK place to save the value of TN.
@@ -551,14 +604,27 @@
 ;;; appropriate. This is also called by SPILL-AND-PACK-LOAD-TN.
 (defun basic-save-tn (tn vop)
   (declare (type tn tn) (type vop vop))
-  (let ((save (tn-save-tn tn)))
-    (cond ((and save (eq (tn-kind save) :save-once))
-           (restore-single-writer-tn tn vop))
-          ((save-single-writer-tn tn)
-           (restore-single-writer-tn tn vop))
-          (t
-           (save-complex-writer-tn tn vop))))
-  (values))
+  (let* ((save (tn-save-tn tn))
+         (node (vop-node vop)))
+    (flet ((restore ()
+             (not
+              (and (sb-c::combination-p node)
+                   ;; Don't restore if the function doesn't return.
+                   (let ((type (sb-c::lvar-fun-type (sb-c::combination-fun node))))
+                     (and (fun-type-p type)
+                          (eq (sb-kernel:fun-type-returns type) *empty-type*)))
+                   #-sb-xc-host
+                   (or
+                    (sb-c::combination-fun-info node)
+                    (policy node (zerop safety)))))))
+      (cond ((and save (eq (tn-kind save) :save-once))
+             (when (restore)
+               (restore-single-writer-tn tn vop)))
+            ((save-single-writer-tn tn)
+             (when (restore)
+               (restore-single-writer-tn tn vop)))
+            (t
+             (save-complex-writer-tn tn vop))))))
 
 ;;; Scan over the VOPs in BLOCK, emiting saving code for TNs noted in
 ;;; the codegen info that are packed into saved SCs.
@@ -830,18 +896,17 @@
 (declaim (end-block))
 
 ;; Misc. utilities
-(declaim (inline unbounded-sc-p))
+(declaim (maybe-inline unbounded-sc-p))
 (defun unbounded-sc-p (sc)
   (eq (sb-kind (sc-sb sc)) :unbounded))
 
 (defun unbounded-tn-p (tn)
+  #-sb-xc-host (declare (inline unbounded-sc-p))
   (unbounded-sc-p (tn-sc tn)))
-(declaim (notinline unbounded-sc-p))
 
 
 ;;;; load TN packing
 
-#-sb-devel
 (declaim (start-block pack-load-tns load-tn-conflicts-in-sc))
 
 ;;; These variables indicate the last location at which we computed
@@ -1166,14 +1231,13 @@
 ;;; not a pure advisory to pack. It allows pack to do some packing it
 ;;; wouldn't have done before.
 (defun pack-load-tn (load-scs op)
-  (declare (type sc-vector load-scs) (type tn-ref op))
+  (declare (type list load-scs) (type tn-ref op))
   (let ((vop (tn-ref-vop op)))
     (compute-live-tns (vop-block vop) vop))
 
   (let* ((tn (tn-ref-tn op))
-         (ptype (tn-primitive-type tn))
-         (scs (svref load-scs (sc-number (tn-sc tn)))))
-    (let ((current-scs scs)
+         (ptype (tn-primitive-type tn)))
+    (let ((current-scs load-scs)
           (allowed ()))
       (loop
         (cond
@@ -1200,7 +1264,7 @@
 ;;; the restriction, we pack a Load-TN and load the operand into it.
 ;;; If a load-tn has already been allocated, we can assume that the
 ;;; restriction is satisfied.
-#-sb-fluid (declaim (inline check-operand-restrictions))
+(declaim (inline check-operand-restrictions))
 (defun check-operand-restrictions (scs ops)
   (declare (list scs) (type (or tn-ref null) ops))
 
@@ -1211,24 +1275,8 @@
     (let ((target (tn-ref-target op))
           (tn (tn-ref-tn op)))
       (when (and target
-                 (not (eq (tn-kind tn) :unused)))
-        (let* ((load-tn (tn-ref-load-tn op))
-               (load-scs (svref (car scs)
-                                (sc-number
-                                 (tn-sc (or load-tn (tn-ref-tn op)))))))
-          (if load-tn
-              (aver (eq load-scs t))
-              (unless (eq load-scs t)
-                (setf (tn-ref-load-tn op)
-                      (pack-load-tn (car scs) op))))))))
-
-  (do ((scs scs (cdr scs))
-       (op ops (tn-ref-across op)))
-      ((null scs))
-    (let ((target (tn-ref-target op))
-          (tn (tn-ref-tn op)))
-      (unless (or target
-                  (eq (tn-kind tn) :unused))
+                 (not (eq (tn-kind tn) :unused))
+                 (tn-primitive-type tn))
         (let* ((load-tn (tn-ref-load-tn op))
                (load-scs (svref (car scs)
                                 (sc-number
@@ -1237,7 +1285,29 @@
               (aver (eq load-scs t))
               (unless (eq load-scs t)
                 (setf (tn-ref-load-tn op)
-                      (pack-load-tn (car scs) op))))))))
+                      (pack-load-tn load-scs op))))))))
+
+  (do ((scs scs (cdr scs))
+       (op ops (tn-ref-across op)))
+      ((null scs))
+    (let ((target (tn-ref-target op))
+          (tn (tn-ref-tn op)))
+      (unless (or target
+                  (eq (tn-kind tn) :unused)
+                  (not (tn-primitive-type tn)))
+        (let* ((load-tn (tn-ref-load-tn op))
+               (load-scs (svref (car scs)
+                                (sc-number
+                                 (tn-sc (or load-tn tn))))))
+          (cond (load-tn
+                 (aver (eq load-scs t)))
+                (t
+                 ;; conditional sc
+                 (when (functionp load-scs)
+                   (setf load-scs (funcall load-scs tn)))
+                 (unless (eq load-scs t)
+                   (setf (tn-ref-load-tn op)
+                         (pack-load-tn load-scs op)))))))))
 
   (values))
 
@@ -1259,7 +1329,6 @@
 
 ;;;; targeting
 
-#-sb-devel
 (declaim (start-block pack pack-tn target-if-desirable
                       ;; needed for pack-iterative
                       pack-wired-tn))
@@ -1315,7 +1384,8 @@
 (declaim (inline %call-with-target-tns))
 (defun %call-with-target-tns (tn callee
                               &key (limit 10) (reads t) (writes t))
-  (declare (type tn tn) (type function callee) (type index limit))
+  (declare (type tn tn) (type function callee) (type index limit)
+           (dynamic-extent callee))
   (flet ((frob-slot (slot-function)
            (declare (type function slot-function))
            (let ((count limit)
@@ -1342,11 +1412,10 @@
                           &rest keys &key limit reads writes)
                          &body body)
   (declare (ignore limit reads writes))
-  (let ((callback (sb-xc:gensym "CALLBACK")))
-    `(flet ((,callback (,target-variable)
-              ,@body))
-       (declare (dynamic-extent #',callback))
-       (%call-with-target-tns ,source-tn #',callback ,@keys))))
+  `(%call-with-target-tns ,source-tn
+                          (lambda (,target-variable)
+                            ,@body)
+                          ,@keys))
 
 (defun find-ok-target-offset (tn sc)
   (declare (type tn tn) (type storage-class sc))
@@ -1479,43 +1548,29 @@
     ;; For non-x86 ports the presence of a save-tn associated with a
     ;; tn is used to identify the old-fp and return-pc tns. It depends
     ;; on the old-fp and return-pc being passed in registers.
-    #-fp-and-pc-standard-save
-    (when (and (not (eq (tn-kind tn) :specified-save))
+    (when (and #-fp-and-pc-standard-save
+               (not (eq (tn-kind tn) :specified-save))
                (conflicts-in-sc original sc offset))
-      (error "~S is wired to a location that it conflicts with." tn))
+      (error "~S is wired to location ~D in SC ~A of kind ~S that it conflicts with."
+             tn offset sc (tn-kind tn)))
 
     ;; Use the above check, but only print a verbose warning. This can
     ;; be helpful for debugging the x86 port.
     #+nil
     (when (and (not (eq (tn-kind tn) :specified-save))
                (conflicts-in-sc original sc offset))
-          (format t "~&* Pack-wired-tn possible conflict:~%  ~
+      (format t "~&* Pack-wired-tn possible conflict:~%  ~
                      tn: ~S; tn-kind: ~S~%  ~
                      sc: ~S~%  ~
                      sb: ~S; sb-name: ~S; sb-kind: ~S~%  ~
                      offset: ~S; end: ~S~%  ~
                      original ~S~%  ~
                      tn-save-tn: ~S; tn-kind of tn-save-tn: ~S~%"
-                  tn (tn-kind tn) sc
-                  sb (sb-name sb) (sb-kind sb)
-                  offset end
-                  original
-                  (tn-save-tn tn) (tn-kind (tn-save-tn tn))))
-
-    ;; On the x86 ports the old-fp and return-pc are often passed on
-    ;; the stack so the above hack for the other ports does not always
-    ;; work. Here the old-fp and return-pc tns are identified by being
-    ;; on the stack in their standard save locations.
-    #+fp-and-pc-standard-save
-    (when (and (not (and
-                     (= (sc-number sc) #.(sc+offset-scn old-fp-passing-offset))
-                     (= offset #.(sc+offset-offset old-fp-passing-offset))))
-               (not (and
-                     (= (sc-number sc) #.(sc+offset-scn return-pc-passing-offset))
-                     (= offset #.(sc+offset-offset return-pc-passing-offset))))
-               (conflicts-in-sc original sc offset))
-      (error "~S is wired to location ~D in SC ~A of kind ~S that it conflicts with."
-             tn offset sc (tn-kind tn)))
+              tn (tn-kind tn) sc
+              sb (sb-name sb) (sb-kind sb)
+              offset end
+              original
+              (tn-save-tn tn) (tn-kind (tn-save-tn tn))))
 
     (unless (eq (sb-kind sb) :unbounded)
       (setf (ldb (byte 1 (truly-the sb-vm:finite-sc-offset offset))
@@ -1564,7 +1619,7 @@
   (let ((path t)) ; dummy initial value
     (labels ((path (lambda)
                (do ((acc '())
-                    (lambda lambda (lambda-parent lambda)))
+                    (lambda lambda (lexenv-lambda (sb-c::lambda-lexenv lambda))))
                    ((null lambda) acc)
                  (push lambda acc)))
              (register-scope (lambda)
@@ -1582,7 +1637,7 @@
       (walk-tn-refs (tn-reads tn))
       (walk-tn-refs (tn-writes tn))
       (if (eql path t)
-          sb-xc:most-positive-fixnum
+          most-positive-fixnum
           (length path)))))
 
 (declaim (type (member :iterative :greedy :adaptive)
@@ -1726,7 +1781,7 @@
       (pack-tns (component))
       (pack-tns (normal))))
 
-  (cond ((and *loop-analyze* *pack-assign-costs*)
+  (cond (*pack-assign-costs*
          ;; Allocate normal TNs, starting with the TNs that are
          ;; heavily used in deep loops (which is taken into account in
          ;; TN spill costs).  Only allocate in finite SCs (i.e. not on
@@ -1735,7 +1790,6 @@
            (do ((tn (ir2-component-normal-tns 2comp) (tn-next tn)))
                ((null tn))
              (unless (or (tn-offset tn)
-                         (eq (tn-kind tn) :more)
                          (unbounded-tn-p tn)
                          (and (sc-save-p (tn-sc tn))  ; SC caller-save, but TN
                               (minusp (tn-cost tn)))) ; lives over many calls
