@@ -63,13 +63,6 @@
            (when barrier (emit-gengc-barrier object nil val-temp t))
            (emit-store (object-slot-ea object offset lowtag) value val-temp)))))
 
-(defun add-symbol-to-remset (vop symbol)
-  (declare (ignorable vop symbol))
-  #+permgen
-  (unless (and (sc-is symbol immediate) (static-symbol-p (tn-value symbol)))
-    (inst push symbol)
-    (invoke-asm-routine 'call 'gc-remember-symbol vop)))
-
 (define-vop (compare-and-swap-slot)
   (:args (object :scs (descriptor-reg) :to :eval)
          (old :scs (descriptor-reg any-reg) #|:target rax|#)
@@ -83,8 +76,10 @@
   (:results (result :scs (descriptor-reg any-reg)))
   (:vop-var vop)
   (:generator 5
-     (when (eq name 'sb-impl::cas-symbol-%info) (add-symbol-to-remset vop object))
-     (emit-gengc-barrier object nil rax (vop-nth-arg 2 vop))
+     (let ((newval-tn-ref (vop-nth-arg 2 vop)))
+       (if (eq name 'sb-impl::cas-symbol-%info)
+           (emit-symbol-write-barrier vop object rax newval-tn-ref)
+           (emit-gengc-barrier object nil rax newval-tn-ref)))
      (move rax old)
      (inst cmpxchg :lock (ea (- (* offset n-word-bytes) lowtag) object) new)
      (move result rax)))
@@ -98,8 +93,12 @@
     (inst mov result unbound-marker-widetag)))
 
 (defun emit-symbol-write-barrier (vop symbol temp newval-tn-ref)
+  (declare (ignorable vop))
+  #+permgen
   (when (require-gengc-barrier-p symbol newval-tn-ref)
-    (add-symbol-to-remset vop symbol))
+    (unless (and (sc-is symbol immediate) (static-symbol-p (tn-value symbol)))
+      (inst push symbol)
+      (invoke-asm-routine 'call 'gc-remember-symbol vop)))
   ;; IMMEDIATE sc means that the symbol is static or immobile.
   ;; Static symbols are roots, and immobile symbols use page fault handling.
   (unless (sc-is symbol immediate)
@@ -150,12 +149,6 @@
                       (object-slot-ea symbol symbol-value-slot other-pointer-lowtag))
       value val-temp)))
 
-(defun emit-lea-symbol-value-slot (ea-tn symbol) ; dest, src
-  (if (sc-is symbol immediate)
-      (inst mov ea-tn (make-fixup (tn-value symbol) :immobile-symbol
-                                  (- (ash symbol-value-slot word-shift) other-pointer-lowtag)))
-      (inst lea ea-tn (object-slot-ea symbol symbol-value-slot other-pointer-lowtag))))
-
 #-sb-thread
 (progn
   (define-vop (symbol-value symbol-global-value)
@@ -652,15 +645,38 @@
               (:arg-types * tagged-num)
               (:results (value :scs (,result-sc)))
               (:result-types ,result-type)
-              (:generator 1 (inst ,inst value (instance-slot-ea object index))))
+              (:generator 1
+                (inst ,inst value (instance-slot-ea object index))))
             (define-vop ()
               (:translate ,(symbolicate "%RAW-INSTANCE-SET/" suffix))
               (:policy :fast-safe)
               (:args (object :scs (descriptor-reg))
                      (index :scs (any-reg immediate))
-                     (value :scs (,result-sc)))
+                     (value :scs (,result-sc ,@(case result-sc
+                                                (single-reg
+                                                 '(fp-single-immediate fp-single-zero))
+                                                (double-reg
+                                                 '(fp-double-zero))
+                                                (complex-single-reg
+                                                 '(fp-complex-single-zero))
+                                                ((unsigned-reg signed-reg)
+                                                 '((immediate (plausible-signed-imm32-operand-p (tn-value tn)))))))))
               (:arg-types * tagged-num ,result-type)
-              (:generator 1 (inst ,inst (instance-slot-ea object index) value))))))
+              (:generator 1
+                ,(if (eq result-sc 'complex-double-reg)
+                     `(inst ,inst (instance-slot-ea object index) value)
+                     `(sc-case value
+                        (,result-sc
+                         (inst ,inst (instance-slot-ea object index) value))
+                        (t
+                         (inst mov ,(case result-sc
+                                      (single-reg :dword)
+                                      (t :qword))
+                               (instance-slot-ea object index)
+                               ,(case result-sc
+                                  (single-reg '(single-float-bits (tn-value value)))
+                                  ((double-reg complex-single-reg) 0)
+                                  (t '(tn-value value))))))))))))
     (def word unsigned-reg unsigned-num mov)
     (def signed-word signed-reg signed-num mov)
     (def single single-reg single-float movss)
@@ -703,7 +719,6 @@
 ;;;;
 
 (defknown %cons-cas-pair (cons t t t t) (values t t))
-(defknown %vector-cas-pair (simple-vector index t t t t) (values t t))
 ;; %INSTANCE-CAS-PAIR only operates on tagged slots (for now)
 (defknown %instance-cas-pair (instance index t t t t) (values t t))
 
@@ -745,6 +760,7 @@
           ;; this is sufficiently confusing that I don't want to try reusing
           ;; one of the other declared temps as the EA for the store barrier.
           (:temporary (:sc unsigned-reg) temp)
+          ;; FIXME: these operand lifetimes are wrong for INDEXEDP = NIL.
           (:temporary (:sc unsigned-reg :offset rax-offset
                        :from (:argument 2) :to (:result 0)) eax)
           (:temporary (:sc unsigned-reg :offset rdx-offset
