@@ -713,13 +713,14 @@
      (let ((l (1- v)))
        (< l v)))
    (member t))
-  (assert-type
-   (lambda (v)
-     (declare (simple-vector v)
-              (optimize (debug 1)))
-     (let ((l (1- (length v))))
-       (< l (length v))))
-   (member t)))
+  ;; (assert-type
+  ;;  (lambda (v)
+  ;;    (declare (simple-vector v)
+  ;;             (optimize (debug 1)))
+  ;;    (let ((l (1- (length v))))
+  ;;      (< l (length v))))
+  ;;  (member t))
+  )
 
 (with-test (:name :sub-sign)
   (assert-type
@@ -1757,7 +1758,8 @@
          (length j))))
    (or null (mod 5))))
 
-(with-test (:name :zero-length-check-bound)
+(with-test (:name :zero-length-check-bound
+            :fails-on (not :sb-unicode))
   (assert (= (count 'sb-kernel:%check-bound
                     (ctu:ir1-named-calls
                      `(lambda (x)
@@ -1767,6 +1769,24 @@
                      nil))
              0)))
 
+(with-test (:name :loop-row-major-aref-check-bound)
+  (assert (= (count 'sb-kernel:%check-bound
+                    (ctu:ir1-named-calls
+                     `(lambda (x)
+                        (declare (type (simple-array t (* *)) x))
+                        (loop for i below (array-total-size x)
+                              do (print (row-major-aref x i))))
+                     nil))
+             0))
+  (assert (= (count 'sb-kernel:%check-bound
+                    (ctu:ir1-named-calls
+                     `(lambda (x)
+                        (declare (type (simple-array t (* *)) x))
+                        (loop for i below (array-total-size x)
+                              do (print (row-major-aref x (1+ i)))))
+                     nil))
+             1)))
+
 (with-test (:name :concatenate-length)
   (assert-type
    (lambda (x y)
@@ -1880,10 +1900,16 @@
    (lambda (m)
      (if (typep (nth-value 1 (truncate m 2.0)) 'double-float)
          m))
-   (or double-float null)))
+   (or double-float null))
+  (assert-type
+   (lambda (x)
+     (declare (integer x))
+     (the (integer 3 83461337) (truncate x -74))
+     x)
+   (integer -6176139011 -222)))
 
 (with-test (:name :ignore-delays
-            :fails-on :arm)
+            :fails-on (or :arm :loongarch64))
   (assert-type
    (lambda (x)
      (declare (optimize debug))
