@@ -174,6 +174,22 @@
     (assert (search "returned 1" output))
     (assert (search "returned 120" output))))
 
+(with-test (:name (trace :encapsulate nil :recursive :threads)
+            :broken-on (:not :arm64))
+  (with-traced-function (trace-fact :encapsulate nil)
+    (let ((threads '()))
+      (dotimes (repeat 4)
+        (push (sb-thread:make-thread
+               (lambda ()
+                 (let ((output
+                         (with-output-to-string (*trace-output*)
+                           (assert (= 120 (trace-fact 5))))))
+                   (assert (search "TRACE-FACT" output))
+                   (assert (search "returned 1" output))
+                   (assert (search "returned 120" output)) )))
+              threads))
+      (mapc #'sb-thread:join-thread threads))))
+
 (defun trace-and-fmakunbound-this (x)
   x)
 
@@ -189,7 +205,7 @@
 (with-test (:name (trace :report nil :smoke))
   (let ((output (with-traced-function (trace-this :report nil)
                   (assert (eq 'ok (trace-this))))))
-    (assert (sequence:emptyp output))))
+    (assert (string= output ""))))
 
 (with-test (:name (trace :report nil :print))
   (let ((output (with-traced-function
@@ -939,7 +955,7 @@
 
 (with-test (:name (:debugger :list-locations)
             ;; there's an extra location on arm for some reason.
-            :fails-on :arm)
+            :fails-on (or :arm :loongarch64))
   (test-debugger
    "ll #'!
     debugger-test-done!"
@@ -963,6 +979,9 @@
    "     (IF (ZEROP N)"
    "         1"
    "         (* N (! (1- N)))))"
+   "7: (IF (ZEROP N)"
+   "       1"
+   "       (* N (! (1- N))))"
    "0]"))
 
 (with-test (:name (:debugger :breakpoint-and-step)
@@ -1314,8 +1333,8 @@
           ;; properly tagged interior pointers. For those which do use LRAs,
           ;; there are at least that many, because we allow pointing to LRAs,
           ;; but they aren't enumerable so we don't know the actual count.
-          (assert (#+(or x86 x86-64 arm64 riscv) =
-                   #-(or x86 x86-64 arm64 riscv) >
+          (assert (#+(or x86 x86-64 arm64 riscv loongarch64) =
+                   #-(or x86 x86-64 arm64 riscv loongarch64) >
                      (loop for ptr from (+ base (* 2 sb-vm:n-word-bytes))
                            below limit count (properly-tagged-p ptr))
                      n))
