@@ -429,7 +429,79 @@ fractional bits."
                  (declare ((double-float 0d0) d))
                  (when (< d 1.0d0)
                    (log d 2.0d0)))
-               (or null (double-float * 0.0d0))))
+               (or null (double-float * 0.0d0)))
+  (assert-type (lambda (d)
+                 (log -5 d))
+               (or (complex single-float) (complex double-float) (member 0.0 0.0d0)))
+  (assert-type (lambda (d)
+                 (declare (double-float d))
+                 (log -5 d))
+               (or (complex double-float) (member 0.0d0)))
+  (assert-type (lambda (d)
+                 (declare ((double-float 10d0) d))
+                 (log -5 d))
+               (complex double-float))
+  (assert-type (lambda (d)
+                 (declare (complex d))
+                 (log -5 d))
+               (or (member 0.0d0 0.0) (complex double-float) (complex single-float)))
+  (assert-type (lambda (d)
+                 (declare ((complex double-float) d))
+                 (log -5 d))
+               (or (member 0.0d0) (complex double-float)))
+  (assert-type (lambda (d)
+                 (declare ((complex single-float) d))
+                 (log -5 d))
+               (or (member 0.0) (complex single-float)))
+  (assert-type (lambda (x y)
+                 (declare ((integer * -1) y))
+                 (log 96 (if x
+                             y
+                             (expt 3434484076828220102 -16))))
+               (or (complex single-float)
+                   (eql #.(log 96 (expt 3434484076828220102 -16)))))
+  (assert-type (lambda (v)
+                 (log 3532504320689493448
+                      (if v
+                          7384646045169235520
+                          51)))
+               (or (eql #.(log 3532504320689493448 7384646045169235520))
+                   (eql #.(log 3532504320689493448 51))))
+  (assert-type (lambda (v)
+                 (LOG 1.9775647e18
+                      (IF v
+                          12
+                          -881536083005615600)))
+               (or (eql #.(log 1.9775647e18 12))
+                   (complex single-float))))
+
+(with-test (:name :log-base-zero)
+  (checked-compile-and-assert
+      ()
+      `(lambda (a b)
+         (declare (single-float a b))
+         (log a b))
+    ((-1.0 0.0) 0.0)
+    ((-1.0 -0.0) 0.0))
+  (checked-compile-and-assert
+      ()
+      `(lambda (a b)
+         (declare (double-float a b))
+         (log a b))
+    ((-1d0 0d0) 0d0)
+    ((-1d0 -0d0) 0d0))
+  (checked-compile-and-assert
+      ()
+      `(lambda (f)
+         (declare (single-float f))
+         (the real (log -3.7739912e17 f)))
+    ((0.0) 0.0))
+  (checked-compile-and-assert
+      ()
+      `(lambda (a b)
+         (declare (single-float a b))
+         (the real (log a b)))
+    ((-3.0 0.0) 0.0)))
 
 (with-test (:name (ftruncate :minus-zeros :one-arg))
   (flet ((f (x) (declare (notinline ftruncate)) (ftruncate x)))
@@ -721,3 +793,23 @@ fractional bits."
                (or (member 0d0) (eql 3.141592653589793d0)))
   (assert (= (phase (opaque-identity -0d0))
              (opaque-identity pi))))
+
+(with-test (:name :log-minus-zero-rational)
+  (sb-int:with-float-traps-masked (:divide-by-zero)
+    (assert (= (log (opaque-identity -0d0) (opaque-identity 2d0))
+               (opaque-identity double-float-negative-infinity)))
+    (assert (= (log (opaque-identity -0f0) (opaque-identity 2f0))
+               (opaque-identity single-float-negative-infinity)))
+    (assert (= (log (opaque-identity -0d0) (opaque-identity 2))
+               (opaque-identity double-float-negative-infinity)))
+    (assert (= (log (opaque-identity -0f0) (opaque-identity 2))
+               (opaque-identity single-float-negative-infinity)))))
+(with-test (:name :ffloor-minus-zero-derive-type)
+  (assert-type (lambda (x)
+                 (declare ((float (-1) (0)) x))
+                 (values (ftruncate x)))
+               (float 0.0 0.0))
+  (assert-type (lambda (x)
+                 (declare ((single-float (0.0) (1.0)) x))
+                 (values (ftruncate x -1.0)))
+               (single-float 0.0 0.0)))
