@@ -191,7 +191,7 @@
                              (+ x0 x1 x6 x7) (+ x2 x3 x4 x5)))))))
 
 (with-test (:name (:nan :comparison)
-            :fails-on (or :sparc))
+            :fails-on (or :sparc :loongarch64))
   (sb-int:with-float-traps-masked (:invalid)
     (macrolet ((test (form)
                  (let ((nform (subst '(/ 0.0 0.0) 'nan form)))
@@ -238,7 +238,7 @@
       (test (not (> nan 1.0))))))
 
 (with-test (:name (:nan :comparison :non-float)
-            :fails-on (or :sparc))
+            :fails-on (or :sparc :loongarch64))
   (sb-int:with-float-traps-masked (:invalid)
     (let ((nan (/ 0.0 0.0))
           (reals (list 0 1 -1 1/2 -1/2 (expt 2 300) (- (expt 2 300))))
@@ -874,3 +874,15 @@
                  #1#
                  (values (ftruncate p 2d0)))
                (eql 5.063089511386214d18)))
+
+(with-test (:name :truncate-big-float)
+  (multiple-value-bind (q r) (truncate (opaque-identity 1f30)
+                                       (opaque-identity 49.12944))
+    (assert (= q 20354393599312814931347243008))
+    (assert (= r #-x86 7.5557864e22 #+x86 4.971228e22))))
+
+(with-test (:name :round-big-float)
+  (multiple-value-bind (q r) (round (opaque-identity (+ most-positive-fixnum 0.6d0))
+                                    (opaque-identity 1d0))
+    (assert (plusp q))
+    (assert (typep r 'double-float))))
