@@ -11,6 +11,8 @@
 ;;;; absolutely no warranty. See the COPYING and CREDITS files for
 ;;;; more information.
 
+(setq *random-state* (make-random-state t))
+
 ;;; As reported by Paul Dietz from his ansi-test suite for gcl, REMOVE
 ;;; malfunctioned when given :START, :END and :FROM-END arguments.
 ;;; Make sure it doesn't happen again.
@@ -237,7 +239,7 @@
              (case safety
                (0 (= speed 0))
                (t t)))
-           (extra-safe (&key speed safety &allow-other-keys)
+           (extra-safe (&key safety &allow-other-keys)
              (>= safety 2))
            (test (type expr &key (filter #'safe))
              (checked-compile-and-assert
@@ -1007,3 +1009,57 @@
               ((simple-string 10) x))
      (coerce x 'list))
    cons))
+
+(with-test (:name :position-if-nil)
+  (assert-type
+   (lambda (f)
+     (position-if f nil))
+   null)
+  (assert-type
+   (lambda (f)
+     (declare (optimize speed))
+     (position-if f nil))
+   null)
+  (assert-type
+   (lambda (f)
+     (find-if f nil))
+   null)
+  (assert-type
+   (lambda (f)
+     (declare (optimize speed))
+     (find-if f nil))
+   null))
+
+(with-test (:name :mismastch-or-null)
+  (assert-type
+   (lambda (a b e)
+     (declare ((or null (eql 4)) e)
+              ((simple-string 9) a))
+     (mismatch a b :start1 5 :end1 e))
+   (or (integer 5 9) null)))
+
+(with-test (:name :reduce-append)
+  (checked-compile-and-assert
+      ()
+      `(lambda (l)
+         (reduce #'append l :initial-value '(1)))
+    (('((a b c))) '(1 a b c) :test #'equal)
+    ((#((a b c))) '(1 a b c) :test #'equal))
+  (checked-compile-and-assert
+      ()
+      `(lambda (l)
+         (reduce #'append l :initial-value '(1) :from-end t))
+    (('((a b c))) '(a b c 1) :test #'equal)
+    ((#((a b c))) '(a b c 1) :test #'equal))
+  (checked-compile-and-assert
+      ()
+      `(lambda (l)
+         (reduce #'append l :end 1 :initial-value '(1)))
+    (('((a b c) (0))) '(1 a b c) :test #'equal)
+    ((#((a b c) (0))) '(1 a b c) :test #'equal))
+  (checked-compile-and-assert
+      ()
+      `(lambda (l)
+         (reduce #'append l :end 1 :initial-value '(1) :from-end t))
+    (('((a b c) (0))) '(a b c 1) :test #'equal)
+    ((#((a b c) (0))) '(a b c 1) :test #'equal)))
